//********************************************************************************************************************************
//*                                                                                                                              *
//*     Filesystem Dialogs - BASS Sampler unit for BASS 2.4  3delite 2006-2008                                                  *
//*     See FilesystemDialogs.txt for details                                                                                    *
//*                                                                                                                              *
//* Checks if there is an ID3v2 "SESC" frame in the (probably) audio file (.mp3 .mp2 .flac) if not, samples the file with BASS   *
//* (if available) then writes this sample back (creates a new) "SESC" frame and ID3v2 if needed.                                *
//*                                                                                                                              *
//* Two licenses are available if you like this library:                                                                         *
//* Shareware License: 50 Euros                                                                                                  *
//* Commercial License: 500 Euros                                                                                                *
//*                                                                                                                              *
//*     http://www.3delite.hu/Object Pascal Developer Resources/filesystemdialogs.html                                           *
//*                                                                                                                              *
//* If you have any questions or enquiries please mail: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit BASSSampler;

interface

Uses SysUtils, Classes, Windows, Helper3delite, ID3v2Library, ThumbDatabase
    , Dialogs
    ;

const
    FILENAME_DLL_BASS     = 'bass.dll';

    BASSVERSION23 = $203;
    BASSVERSION24 = $204;

    BASS_POS_BYTE           = 0;

    BASS_ERROR_ALREADY      = 14;   // already initialized/paused/whatever

    BASS_CONFIG_BUFFER      = 0;

    BASS_STREAM_PRESCAN     = $20000; // enable pin-point seeking (MP3/MP2/MP1)
    BASS_MP3_SETPOS         = BASS_STREAM_PRESCAN;
    BASS_STREAM_AUTOFREE	= $40000; // automatically free the stream when it stop/ends
    BASS_STREAM_RESTRATE	= $80000; // restrict the download rate of internet file streams
    BASS_STREAM_BLOCK       = $100000;// download/play internet file stream in small blocks
    BASS_STREAM_DECODE      = $200000;// don't play the stream, only decode (BASS_ChannelGetData)
    BASS_STREAM_STATUS      = $800000;// give server status info (HTTP/ICY tags) in DOWNLOADPROC

    BASS_UNICODE            = $80000000;

    BASS_SAMPLE_FLOAT       = 256; // 32-bit floating-point
    BASS_SAMPLE_MONO        = 2;   // mono, else stereo
    BASS_SAMPLE_LOOP        = 4;   // looped
    BASS_SAMPLE_3D          = 8;   // 3D functionality enabled
    BASS_SAMPLE_SOFTWARE    = 16;  // it's NOT using hardware mixing
    BASS_SAMPLE_FX          = 128; // old implementation of DX8 effects are enabled

    BASS_MUSIC_FLOAT        = BASS_SAMPLE_FLOAT; // 32-bit floating-point
    BASS_MUSIC_MONO         = BASS_SAMPLE_MONO; // force mono mixing (less CPU usage)
    BASS_MUSIC_LOOP         = BASS_SAMPLE_LOOP; // loop music
    BASS_MUSIC_3D           = BASS_SAMPLE_3D; // enable 3D functionality
    BASS_MUSIC_FX           = BASS_SAMPLE_FX; // enable old implementation of DX8 effects
    BASS_MUSIC_AUTOFREE     = BASS_STREAM_AUTOFREE; // automatically free the music when it stop/ends
    BASS_MUSIC_DECODE       = BASS_STREAM_DECODE; // don't play the music, only decode (BASS_ChannelGetData)
    BASS_MUSIC_PRESCAN      = BASS_STREAM_PRESCAN; // calculate playback length
    BASS_MUSIC_CALCLEN      = BASS_MUSIC_PRESCAN;
    BASS_MUSIC_RAMP         = $200;  // normal ramping
    BASS_MUSIC_RAMPS        = $400;  // sensitive ramping
    BASS_MUSIC_SURROUND     = $800;  // surround sound
    BASS_MUSIC_SURROUND2    = $1000; // surround sound (mode 2)
    BASS_MUSIC_FT2MOD       = $2000; // play .MOD as FastTracker 2 does
    BASS_MUSIC_PT1MOD       = $4000; // play .MOD as ProTracker 1 does
    BASS_MUSIC_NONINTER     = $10000; // non-interpolated mixing
    BASS_MUSIC_POSRESET     = $8000; // stop all notes when moving position
    BASS_MUSIC_POSRESETEX   = $400000; // stop all notes and reset bmp/etc when moving position
    BASS_MUSIC_STOPBACK     = $80000; // stop the music on a backwards jump effect
    BASS_MUSIC_NOSAMPLE     = $100000; // don't load the samples

    BASS_SYNC_END                     = 2;

    // BASS_ChannelIsActive return values
    BASS_ACTIVE_STOPPED = 0;
    BASS_ACTIVE_PLAYING = 1;
    BASS_ACTIVE_STALLED = 2;
    BASS_ACTIVE_PAUSED  = 3;

    // BASS_ChannelGetData flags
    BASS_DATA_AVAILABLE = 0;        // query how much data is buffered
    BASS_DATA_FLOAT     = $40000000; // flag: return floating-point sample data
    BASS_DATA_FFT512    = $80000000; // 512 sample FFT
    BASS_DATA_FFT1024   = $80000001; // 1024 FFT
    BASS_DATA_FFT2048   = $80000002; // 2048 FFT
    BASS_DATA_FFT4096   = $80000003; // 4096 FFT
    BASS_DATA_FFT_INDIVIDUAL = $10; // FFT flag: FFT for each channel, else all combined
    BASS_DATA_FFT_NOWINDOW = $20;   // FFT flag: no Hanning window

type
    TSampleCache = Array of Byte;

type
    HPLUGIN = DWORD;      // Plugin handle
    HSAMPLE = DWORD;      // sample handle
    FLOAT = Single;
    QWORD = Int64;
    HSYNC = DWORD;        // synchronizer handle

    BASS23_CHANNELINFO = record
	    freq: DWORD;        // default playback rate
	    chans: DWORD;       // channels
	    flags: DWORD;       // BASS_SAMPLE/STREAM/MUSIC/SPEAKER flags
	    ctype: DWORD;       // type of channel
	    origres: DWORD;     // original resolution
	    plugin: HPLUGIN;    // plugin
    end;

    BASS24_CHANNELINFO = record
        freq: DWORD;        // default playback rate
        chans: DWORD;       // channels
        flags: DWORD;       // BASS_SAMPLE/STREAM/MUSIC/SPEAKER flags
        ctype: DWORD;       // type of channel
        origres: DWORD;     // original resolution
        plugin: HPLUGIN;    // plugin
        sample: HSAMPLE;    // sample
        filename: PChar; // filename
    end;

    SYNCPROC = procedure(handle: HSYNC; channel, data: DWORD; user: DWORD); stdcall;

    procedure CheckBASSAvailability(LoadBASS: Boolean);
    function InitializeBASS(Device: Integer; freq: DWORD; win: HWND): LongBool;

    function StreamLoadSESC(Tag: Pointer): TSampleCache;
    function ID3v2_SetSESC(Tag: Pointer; SESC: TSampleCache; Force: Boolean): Boolean;

    function ID3v2_GetSEBR(Tag: Pointer): Extended;
    function ID3v2_GetAPIC(Tag: Pointer; Index: Cardinal; var APIC: TStream): Boolean;
    function GetID3Info(FileName, ItemName: String): String;

    function BASS_Init(device: Integer; freq, flags: DWORD; win: HWND; clsid: PGUID): BOOL;
    function BASS_SetConfig(option, value: DWORD): DWORD;
    function BASS_MusicLoad(mem: BOOL; f: Pointer; offset: QWORD; length, flags, freq: DWORD): DWORD;
    function BASS_StreamCreateFile(mem: BOOL; f: Pointer; offset, length: QWORD; flags: DWORD): DWORD;
    function BASS_ChannelGetLength(handle: DWORD): Int64;
    function BASS_ChannelGetInfo(handle: DWORD; var info: BASS24_CHANNELINFO):BOOL;
    function BASS_ChannelIsActive(handle: DWORD): DWORD;
    function BASS_ChannelGetData(handle: DWORD; buffer: Pointer; length: DWORD): DWORD;
    function BASS_ChannelGetPosition(handle: DWORD): QWORD;
    function BASS_ChannelSetPosition(handle: DWORD; pos: QWORD): BOOL;
    function BASS_ChannelBytes2Seconds(handle: DWORD; pos: QWORD): FLOAT;
    function BASS_ChannelSetSync(handle: DWORD; type_: DWORD; param: QWORD; proc: SYNCPROC; user: Pointer): HSYNC;
    function BASS_MusicFree(Channel: Cardinal): Bool;
    function BASS_StreamFree(Channel: Cardinal): Bool;
    function BASS_GetDevice: DWORD;
    function BASS_ChannelPlay(handle: DWORD; restart: BOOL): BOOL;
    function BASS_ChannelStop(handle: DWORD): BOOL;
    function BASS_ChannelPause(handle: DWORD): BOOL;
    function BASS_ErrorGetCode: LongInt;

var
  BASS_Available: Boolean = False;
  BASS_ActiveDevice: Integer;
  BASS_LibHandle: THandle;
  BASS_Version: DWORD;
  BASS_GetVersion: function: DWORD; stdcall;

  t_BASS23_Init: function(device: Integer; freq, flags: DWORD; win: HWND; clsid: PGUID): BOOL; stdcall;
  t_BASS23_SetConfig: function(option, value: DWORD): DWORD; stdcall;
  t_BASS23_MusicLoad: function(mem: BOOL; f: Pointer; offset, length, flags, freq: DWORD): DWORD; stdcall;
  t_BASS23_StreamCreateFile: function(mem: BOOL; f: Pointer; offset, length, flags: DWORD): DWORD; stdcall;
  t_BASS23_ChannelGetLength: function(handle: DWORD): Int64; stdcall;
  t_BASS23_ChannelGetInfo: function(handle: DWORD; var info: BASS23_CHANNELINFO):BOOL; stdcall;
  t_BASS23_ChannelIsActive: function(handle: DWORD): DWORD; stdcall;
  t_BASS23_ChannelGetData: function(handle: DWORD; buffer: Pointer; length: DWORD): DWORD; stdcall;
  t_BASS23_ChannelGetPosition: function(handle: DWORD): QWORD; stdcall;
  t_BASS23_ChannelSetPosition: function(handle: DWORD; pos: QWORD): BOOL; stdcall;
  t_BASS23_ChannelBytes2Seconds: function(handle: DWORD; pos: QWORD): FLOAT; stdcall;
  t_BASS23_ChannelSetSync: function(handle: DWORD; stype: DWORD; param: QWORD; proc: SYNCPROC; user: Pointer): HSYNC; stdcall;
  t_BASS23_MusicFree: function(Channel: Cardinal): Bool; stdcall;
  t_BASS23_StreamFree: function(Channel: Cardinal): Bool; stdcall;
  t_BASS23_GetDevice: function: DWORD; stdcall;
  t_BASS23_ChannelPlay: function(handle: DWORD; restart: BOOL): BOOL; stdcall;
  t_BASS23_ChannelStop: function(handle: DWORD): BOOL; stdcall;
  t_BASS23_ChannelPause: function(handle: DWORD): BOOL; stdcall;
  t_BASS23_ErrorGetCode: function: LongInt; stdcall;

  t_BASS24_Init: function(device: Integer; freq, flags: DWORD; win: HWND; clsid: PGUID): BOOL; stdcall;
  t_BASS24_SetConfig: function(option, value: DWORD): DWORD; stdcall;
  t_BASS24_MusicLoad: function(mem: BOOL; f: Pointer; offset: QWORD; length, flags, freq: DWORD): DWORD; stdcall;
  t_BASS24_StreamCreateFile: function(mem: BOOL; f: Pointer; offset, length: QWORD; flags: DWORD): DWORD; stdcall;
  t_BASS24_ChannelGetLength: function(handle: DWORD; mode: DWORD): Int64; stdcall;
  t_BASS24_ChannelGetInfo: function(handle: DWORD; var info: BASS24_CHANNELINFO):BOOL; stdcall;
  t_BASS24_ChannelIsActive: function(handle: DWORD): DWORD; stdcall;
  t_BASS24_ChannelGetData: function(handle: DWORD; buffer: Pointer; length: DWORD): DWORD; stdcall;
  t_BASS24_ChannelGetPosition: function(handle: DWORD; mode: DWORD): QWORD; stdcall;
  t_BASS24_ChannelSetPosition: function(handle: DWORD; pos: QWORD; mode: DWORD): BOOL; stdcall;
  t_BASS24_ChannelBytes2Seconds: function(handle: DWORD; pos: QWORD): FLOAT; stdcall;
  t_BASS24_ChannelSetSync: function(handle: DWORD; type_: DWORD; param: QWORD; proc: SYNCPROC; user: Pointer): HSYNC; stdcall;
  t_BASS24_MusicFree: function(Channel: Cardinal): Bool; stdcall;
  t_BASS24_StreamFree: function(Channel: Cardinal): Bool; stdcall;
  t_BASS24_GetDevice: function: DWORD; stdcall;
  t_BASS24_ChannelPlay: function(handle: DWORD; restart: BOOL): BOOL; stdcall;
  t_BASS24_ChannelStop: function(handle: DWORD): BOOL; stdcall;
  t_BASS24_ChannelPause: function(handle: DWORD): BOOL; stdcall;
  t_BASS24_ErrorGetCode: function: LongInt; stdcall;

implementation

//Uses FileDialog;

procedure CheckBASSAvailability(LoadBASS: Boolean);
begin

    BASS_LibHandle := GetModuleHandle(PChar(FILENAME_DLL_BASS));

    if (BASS_LibHandle = 0)
    AND LoadBASS
        then BASS_LibHandle := SafeLoadLibrary(PChar(FILENAME_DLL_BASS));

    if BASS_LibHandle <> 0 then begin
        BASS_GetVersion := GetProcAddress(BASS_LibHandle, PChar('BASS_GetVersion'));

        t_BASS23_Init := GetProcAddress(BASS_LibHandle, PChar('BASS_Init'));
        t_BASS23_SetConfig := GetProcAddress(BASS_LibHandle, PChar('BASS_SetConfig'));
        t_BASS23_MusicLoad := GetProcAddress(BASS_LibHandle, PChar('BASS_MusicLoad'));
        t_BASS23_StreamCreateFile := GetProcAddress(BASS_LibHandle, PChar('BASS_StreamCreateFile'));
        t_BASS23_ChannelGetLength := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelGetLength'));
        t_BASS23_ChannelGetInfo := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelGetInfo'));
        t_BASS23_ChannelIsActive := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelIsActive'));
        t_BASS23_ChannelGetData := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelGetData'));
        t_BASS23_ChannelGetPosition := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelGetPosition'));
        t_BASS23_ChannelSetPosition := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelSetPosition'));
        t_BASS23_ChannelBytes2Seconds := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelBytes2Seconds'));
        t_BASS23_ChannelSetSync := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelSetSync'));
        t_BASS23_MusicFree := GetProcAddress(BASS_LibHandle, PChar('BASS_MusicFree'));
        t_BASS23_StreamFree := GetProcAddress(BASS_LibHandle, PChar('BASS_StreamFree'));
        t_BASS23_GetDevice := GetProcAddress(BASS_LibHandle, PChar('BASS_GetDevice'));
        t_BASS23_ChannelPlay := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelPlay'));
        t_BASS23_ChannelStop := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelStop'));
        t_BASS23_ChannelPause := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelPause'));
        t_BASS23_ErrorGetCode := GetProcAddress(BASS_LibHandle, PChar('BASS_ErrorGetCode'));

        t_BASS24_Init := GetProcAddress(BASS_LibHandle, PChar('BASS_Init'));
        t_BASS24_SetConfig := GetProcAddress(BASS_LibHandle, PChar('BASS_SetConfig'));
        t_BASS24_MusicLoad := GetProcAddress(BASS_LibHandle, PChar('BASS_MusicLoad'));
        t_BASS24_StreamCreateFile := GetProcAddress(BASS_LibHandle, PChar('BASS_StreamCreateFile'));
        t_BASS24_ChannelGetLength := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelGetLength'));
        t_BASS24_ChannelGetInfo := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelGetInfo'));
        t_BASS24_ChannelIsActive := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelIsActive'));
        t_BASS24_ChannelGetData := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelGetData'));
        t_BASS24_ChannelGetPosition := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelGetPosition'));
        t_BASS24_ChannelSetPosition := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelSetPosition'));
        t_BASS24_ChannelBytes2Seconds := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelBytes2Seconds'));
        t_BASS24_ChannelSetSync := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelSetSync'));
        t_BASS24_MusicFree := GetProcAddress(BASS_LibHandle, PChar('BASS_MusicFree'));
        t_BASS24_StreamFree := GetProcAddress(BASS_LibHandle, PChar('BASS_StreamFree'));
        t_BASS24_GetDevice := GetProcAddress(BASS_LibHandle, PChar('BASS_GetDevice'));
        t_BASS24_ChannelPlay := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelPlay'));
        t_BASS24_ChannelStop := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelStop'));
        t_BASS24_ChannelPause := GetProcAddress(BASS_LibHandle, PChar('BASS_ChannelPause'));
        t_BASS24_ErrorGetCode := GetProcAddress(BASS_LibHandle, PChar('BASS_ErrorGetCode'));
    end;

    if (BASS_LibHandle <> 0)
        then BASS_Available := True
        else BASS_Available := False;

    BASS_Version := HIWORD(BASS_GetVersion);

    if BASS_Available then begin
        case BASS_Version of
            BASSVERSION23: BASS_ActiveDevice := t_BASS23_GetDevice;
            BASSVERSION24: BASS_ActiveDevice := t_BASS24_GetDevice;
            else begin
                BASS_ActiveDevice := t_BASS24_GetDevice;
            end;
        end;
    end;

end;

function InitializeBASS(Device: Integer; freq: DWORD; win: HWND): LongBool;
var
  Error: Integer;
begin
    Result := False;
    case BASS_Version of
        BASSVERSION23: begin
            if t_BASS23_Init(Device, freq, 0, win, nil) then begin
                BASS_ActiveDevice := t_BASS23_GetDevice;
                Result := True;
                t_BASS23_SetConfig(BASS_CONFIG_BUFFER, 1000 * 2);
            end;
            Error := t_BASS23_ErrorGetCode;
            if Error = BASS_ERROR_ALREADY then begin
                Result := True;
            end;
        end;
        BASSVERSION24: begin
            if t_BASS24_Init(Device, freq, 0, win, nil) then begin
                BASS_ActiveDevice := t_BASS24_GetDevice;
                Result := True;
                t_BASS24_SetConfig(BASS_CONFIG_BUFFER, 1000 * 2);
            end;
            Error := t_BASS24_ErrorGetCode;
            if Error = BASS_ERROR_ALREADY then begin
                Result := True;
            end;
        end
        else begin
            if t_BASS24_Init(Device, freq, 0, win, nil) then begin
                BASS_ActiveDevice := t_BASS24_GetDevice;
                Result := True;
                t_BASS24_SetConfig(BASS_CONFIG_BUFFER, 1000 * 2);
            end;
            Error := t_BASS24_ErrorGetCode;
            if Error = BASS_ERROR_ALREADY then begin
                Result := True;
            end;
        end;
    end;

end;

function StreamLoadSESC(Tag: Pointer): TSampleCache;
var
  SESCStream: TStream;
  i: integer;
  Frame: AnsiString;
begin
    SetLength(Result, 0);
    if NOT ID3v2_Loaded(Tag)
        then Exit;
    SESCStream := TMemoryStream.Create;
    try
        for i := 1 to ID3v2_FrameCount(Tag) do begin
            Frame := ID3v2_GetFrameID(Tag, i);
            if Length(Frame) > 4
                then Frame := Copy(Frame, 0, 4);
            if Frame = 'SESC' then begin
                ID3v2_DeCompressFrame(Tag, i);
                ID3v2_GetFrameData(Tag, i, SESCStream);
                SetLength(Result, SESCStream.Size);
                SESCStream.Seek(0, soFromBeginning);
                {$RANGECHECKS OFF}
                SESCStream.ReadBuffer(Pointer(Result)^, SESCStream.Size);
                Break;
            end;
        end;
    finally
        SESCStream.Free;
    end;
end;

function ID3v2_SetSESC(Tag: Pointer; SESC: TSampleCache; Force: Boolean): Boolean;
var
  SESCStream: TStream;
  i, tempIntv: integer;
  Index: Cardinal;
  Frame: AnsiString;
begin
    Result := False;
{$RANGECHECKS OFF}
    if ID3v2_Loaded(Tag)
    OR Force
    then begin
        SESCStream := TMemoryStream.Create;
        try
            for i := 1 to ID3v2_FrameCount(Tag) do begin
                Frame := ID3v2_GetFrameID(Tag, i);
                if Length(Frame) > 4
                    then Frame := Copy(Frame, 0, 4);
                if Frame = 'SESC' then begin
                    ID3v2_DeleteFrame(Tag, i);
                    Break;
                end;
                frame := '';
            end;

            Frame := 'SESC';
            Index := ID3v2_CreateFrame(Tag, PANSIChar(Frame));
            for i := 0 to High(SESC) - 1
                do SESCStream.Write(SESC[i], SizeOf(SESC[i]));
            ID3v2_SetFrameData(Tag, Index, SESCStream);
            Result := ID3v2_CompressFrame(Tag, Index);
        finally
            SESCStream.Free;
        end;
    end;
end;

function ID3v2_GetSEBR(Tag: Pointer): Extended;
var
  TempMStream: TStream;
  fIndex: Integer;
  SEBR: Extended;
begin
    try
        try
            SEBR := 0;
            Result := 0;
            TempMStream := nil;
            fIndex := ID3v2_CheckFrameExists(Tag, 'SEBR');
            if fIndex > 0 then begin
                TempMStream := TMemoryStream.Create;
                ID3v2_DeCompressFrame(Tag, fIndex);
                ID3v2_GetFrameData(Tag, fIndex, TempMStream);
                TempMStream.Seek(0, soFromBeginning);
                TempMStream.Read(SEBR, 10);
                Result := SEBR;
            end;
        except
            //*
        end;
    finally
        FreeAndNil(TempMStream);
    end;
end;

function ID3v2_GetAPIC(Tag: Pointer; Index: Cardinal; var APIC: TStream): Boolean;
var
  TempMStream: TStream;
  Pos: Cardinal;
  Buffer: Byte;
begin
    try
        try
            Result := False;
            Pos := 0;
            TempMStream := nil;
            APIC.Size := 0;
            if Index > 0 then begin
                TempMStream := TMemoryStream.Create;
                ID3v2_DeCompressFrame(Tag, Index);
                ID3v2_GetFrameData(Tag, Index, TempMStream);
                TempMStream.Seek(0, soFromBeginning);
                TempMStream.ReadBuffer(Buffer, 1);
                if Buffer <> $FF then begin
                    //Inc(Pos);
                    repeat
                        TempMStream.ReadBuffer(Buffer, 1);
                        Inc(Pos);
                    until Buffer = $FF;
                end;
                TempMStream.Seek(Pos, soFromBeginning);
                APIC.CopyFrom(TempMStream, TempMStream.Size - Pos);
                APIC.Seek(0, soFromBeginning);
                Result := True;
            end;
        except
            //*
        end;
    finally
        FreeAndNil(TempMStream);
    end;
end;

function GetID3Info(FileName, ItemName: String): String;
var
  Tagv1: Pointer;
  Tagv2: Pointer;
  Artist, Title, Album, Year, TLEN, SEBR: String;
  Fext: String;
  SEBRValue: Extended;
  ANSIFileName: AnsiString;
  TagStream: TStream;
begin

    Result := '';

    Fext := AnsiUpperCase(ExtractFileExt(FileName));

    if NOT (
       (Fext = '.MP1')
    OR (Fext = '.MP2')
    OR (Fext = '.MP3')
    OR (Fext = '.MP4')
    OR (Fext = '.M4A')
    OR (Fext = '.WAV')
    OR (Fext = '.MPC')
    OR (Fext = '.AAC')
    OR (Fext = '.APE')
    OR (Fext = '.VQF')
    OR (Fext = '.FLAC')
            )
        then Exit;

    Tagv1 := ID3v1_Create;
    Tagv2 := ID3v2_Create;
    try
        try
            //* Load Tags
            ANSIFileName := AnsiString(FileName);
            ID3v1_Load(Tagv1, PANSIChar(ANSIFileName));

            TagStream := TFileStream.Create(FileName, fmOpenRead OR fmShareDenyNone);
            ID3v2_LoadFromStream(Tagv2, TagStream);
            FreeAndNil(TagStream);

            if NOT ID3v2_Loaded(Tagv2)
                then Exit;

            //* Artist
            Artist := ID3v2_GetUnicodeText(Tagv2, PANSIChar('TPE1'));
            if Trim(Artist) = ''
                then if Trim(ID3v1_GetArtist(Tagv1)) <> ''
                    then Artist := Trim(ID3v1_GetArtist(Tagv1));

            //* Title
            Title := ID3v2_GetUnicodeText(Tagv2, PANSIChar('TIT2'));
            if Trim(Title) = ''
                then if Trim(ID3v1_GetTitle(Tagv1)) <> ''
                    then Title := Trim(ID3v1_GetTitle(Tagv1));

            //* Album
            Album := ID3v2_GetUnicodeText(Tagv2, PANSIChar('TALB'));
            if Trim(Album) = ''
                then if Trim(ID3v1_GetAlbum(Tagv1)) <> ''
                    then Album := Trim(ID3v1_GetAlbum(Tagv1));

            //* Year
            Year := ID3v2_GetUnicodeText(Tagv2, PANSIChar('TYER'));
            if Trim(Year) = ''
                then if Trim(ID3v1_GetYear(Tagv1)) <> ''
                    then Album := Trim(ID3v1_GetYear(Tagv1));

            //* Length
            TLEN := ID3v2_GetUnicodeText(Tagv2, PANSIChar('TLEN'));
            if Trim(TLEN) = '' then begin
                TLEN := '';
            end else begin
                try
                    TLEN := mSec2Time(StrToInt(TLEN));
                except
                    TLEN := '';
                end;
            end;

            //* Bitrate
            SEBRValue := ID3v2_GetSEBR(Tagv2);
            SEBR := FloatToStrF(SEBRValue, ffFixed, 4, 2);
            if (Trim(SEBR) = '')
            OR (SEBRValue = 0)
            OR (SEBR = '0')
            then begin
                SEBR := '';
            end else begin
                try
                    SEBR := SEBR + 'kbps';
                except
                    SEBR := '';
                end;
            end;

        except
            //*
        end;
    finally

        //* Add Artist
        if Artist <> ''
            then Result := Artist;

        //* Add Title
        if Title <> '' then begin
            //* If no previous data add just Title
            if Result <> ''
                then Result := Result + ' - ' + Title
                else Result := Title;
        end;

        //* If Year exists
        if year <> ''
            then if Album <> ''
                //* And if Album exists too join them together
                then Album := Year + ': ' + Album
                //* If not, only year
                else Album := Year;
        //* Add Album and/or year to Artist and Title
        if Result <> '' then begin
            //* If we have Album and/or year add them to Artist and Title
            if Album <> ''
                then Result := Result + ' (' + Album  + ')';
        //* If no Artist and Title result is only Album and/or year
        end else Result := Album;

        //* If nothing found write filename
        if Result = ''
            then Result := ItemName;

        //* If we have result and more data carriege return
        if (Result <> '')
        AND ((SEBR <> '') OR (TLEN <> ''))
                then Result := Result + (#13#10);

        //* If we have bit rate value add it
        if SEBR <> ''
            then Result := Result + SEBR;

        //* If we have playtime value
        if TLEN <> '' then begin
            //* if we don't have so far data use only TLEN
            if Result = ''
                then Result := '[' + TLEN + ']'
                //* if we have so far data add it to them
                else begin
                    if SEBR = ''
                        //* If no SEBR, TLEN starts at the beginning
                        then Result := Result + '[' + TLEN + ']'
                        //* TLEN starts after SEBR
                        else Result := Result + ' [' + TLEN + ']';
                end;
        end;

        SEBRValue := 0;
        ID3v1_Free(Tagv1);
        ID3v2_Free(Tagv2);
    end;
end;

function BASS_Init(device: Integer; freq, flags: DWORD; win: HWND; clsid: PGUID): BOOL;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_Init(device, freq, flags, win, clsid);
        BASSVERSION24: Result := t_BASS24_Init(device, freq, flags, win, clsid);
        else begin
            Result := t_BASS24_Init(device, freq, flags, win, clsid);
        end;
    end;
end;

function BASS_SetConfig(option, value: DWORD): DWORD;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_SetConfig(option, value);
        BASSVERSION24: Result := t_BASS24_SetConfig(option, value);
        else begin
            Result := t_BASS24_SetConfig(option, value);
        end;
    end;
end;

function BASS_MusicLoad(mem: BOOL; f: Pointer; offset: QWORD; length, flags, freq: DWORD): DWORD;
var
    Channel: Cardinal;
    //Error: Integer;
    //Address: Pointer;
begin

    //Address := Pointer(@t_BASS24_MusicLoad);
    //Showmessage(IntToHex(Integer(Address), 8));

    case BASS_Version of
        BASSVERSION23: begin
            Channel := t_BASS23_MusicLoad(mem, f, offset, length, flags, freq);
        end;
        BASSVERSION24: begin
            Channel := t_BASS24_MusicLoad(mem, f, offset, length, flags, freq);
        end
        else begin
            Channel := t_BASS24_MusicLoad(mem, f, offset, length, flags, freq);
        end;
    end;
    {
    Error := t_BASS24_ErrorGetCode;

    if Error <> 0 then begin
        Showmessage('Error: ' + IntToStr(Error));
    end;
    }

    Result := Channel;

    //Showmessage('Channel: ' + IntToStr(Result));

end;

function BASS_StreamCreateFile(mem: BOOL; f: Pointer; offset, length: QWORD; flags: DWORD): DWORD;
var
    Channel: Cardinal;
    //Error: Integer;
begin
    case BASS_Version of
        BASSVERSION23: begin
            Channel := t_BASS23_StreamCreateFile(mem, f, offset, length, flags);
        end;
        BASSVERSION24: begin
            Channel := t_BASS24_StreamCreateFile(mem, f, offset, length, flags);
        end
        else begin
            Channel := t_BASS24_StreamCreateFile(mem, f, offset, length, flags);
        end;
    end;
    {
    Error := t_BASS24_ErrorGetCode;

    if Error <> 0 then begin
        Showmessage('Error: ' + IntToStr(Error));
    end;
    }
    Result := Channel;

    //Showmessage('Channel: ' + IntToStr(Result));

end;

function BASS_ChannelGetLength(handle: DWORD): Int64;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_ChannelGetLength(handle);
        BASSVERSION24: Result := t_BASS24_ChannelGetLength(handle, BASS_POS_BYTE);
        else begin
            Result := t_BASS24_ChannelGetLength(handle, BASS_POS_BYTE);
        end;
    end;
end;

function BASS_ChannelGetInfo(handle: DWORD; var info: BASS24_CHANNELINFO):BOOL;
var
    ChannelInfo: BASS23_CHANNELINFO;
begin
    case BASS_Version of
        BASSVERSION23: begin
            ZeroMemory(@ChannelInfo, SizeOf(BASS23_CHANNELINFO));
            ChannelInfo.freq := info.freq;
            ChannelInfo.chans := info.chans;
            ChannelInfo.flags := info.flags;
            ChannelInfo.ctype := info.ctype;
            ChannelInfo.origres := info.origres;
            ChannelInfo.plugin := info.plugin;
            Result := t_BASS23_ChannelGetInfo(handle, ChannelInfo);
            info.freq := ChannelInfo.freq;
            info.chans := ChannelInfo.chans;
            info.flags := ChannelInfo.flags;
            info.ctype := ChannelInfo.ctype;
            info.origres := ChannelInfo.origres;
            info.plugin := ChannelInfo.plugin;
        end;
        BASSVERSION24: Result := t_BASS24_ChannelGetInfo(handle, info);
        else begin
            Result := t_BASS24_ChannelGetInfo(handle, info);
        end;
    end;
end;

function BASS_ChannelIsActive(handle: DWORD): DWORD;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_ChannelIsActive(handle);
        BASSVERSION24: Result := t_BASS24_ChannelIsActive(handle);
        else begin
            Result := t_BASS24_ChannelIsActive(handle);
        end;
    end;
end;

function BASS_ChannelGetData(handle: DWORD; buffer: Pointer; length: DWORD): DWORD;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_ChannelGetData(handle, buffer, length);
        BASSVERSION24: Result := t_BASS24_ChannelGetData(handle, buffer, length);
        else begin
            Result := t_BASS24_ChannelGetData(handle, buffer, length);
        end;
    end;
end;

function BASS_ChannelGetPosition(handle: DWORD): QWORD;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_ChannelGetPosition(handle);
        BASSVERSION24: Result := t_BASS24_ChannelGetPosition(handle, BASS_POS_BYTE);
        else begin
            Result := t_BASS24_ChannelGetPosition(handle, BASS_POS_BYTE);
        end;
    end;
end;

function BASS_ChannelSetPosition(handle: DWORD; pos: QWORD): BOOL;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_ChannelSetPosition(handle, pos);
        BASSVERSION24: Result := t_BASS24_ChannelSetPosition(handle, pos, BASS_POS_BYTE);
        else begin
            Result := t_BASS24_ChannelSetPosition(handle, pos, BASS_POS_BYTE);
        end;
    end;
end;

function BASS_ChannelBytes2Seconds(handle: DWORD; pos: QWORD): FLOAT;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_ChannelBytes2Seconds(handle, pos);
        BASSVERSION24: Result := t_BASS24_ChannelBytes2Seconds(handle, pos);
        else begin
            Result := t_BASS24_ChannelBytes2Seconds(handle, pos);
        end;
    end;
end;

function BASS_ChannelSetSync(handle: DWORD; type_: DWORD; param: QWORD; proc: SYNCPROC; user: Pointer): HSYNC;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_ChannelSetSync(handle, type_, param, proc, user);
        BASSVERSION24: Result := t_BASS24_ChannelSetSync(handle, type_, param, proc, user);
        else begin
            Result := t_BASS24_ChannelSetSync(handle, type_, param, proc, user);
        end;
    end;
end;

function BASS_MusicFree(Channel: Cardinal): Bool;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_MusicFree(Channel);
        BASSVERSION24: Result := t_BASS24_MusicFree(Channel);
        else begin
            Result := t_BASS24_MusicFree(Channel);
        end;
    end;
end;

function BASS_StreamFree(Channel: Cardinal): Bool;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_StreamFree(Channel);
        BASSVERSION24: Result := t_BASS24_StreamFree(Channel);
        else begin
            Result := t_BASS24_StreamFree(Channel);
        end;
    end;
end;
function BASS_GetDevice: DWORD;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_GetDevice;
        BASSVERSION24: Result := t_BASS24_GetDevice;
        else begin
            Result := t_BASS24_GetDevice;
        end;
    end;
end;

function BASS_ChannelPlay(handle: DWORD; restart: BOOL): BOOL;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_ChannelPlay(handle, restart);
        BASSVERSION24: Result := t_BASS24_ChannelPlay(handle, restart);
        else begin
            Result := t_BASS24_ChannelPlay(handle, restart);
        end;
    end;
end;

function BASS_ChannelStop(handle: DWORD): BOOL;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_ChannelStop(handle);
        BASSVERSION24: Result := t_BASS24_ChannelStop(handle);
        else begin
            Result := t_BASS24_ChannelStop(handle);
        end;
    end;
end;

function BASS_ChannelPause(handle: DWORD): BOOL;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_ChannelPause(handle);
        BASSVERSION24: Result := t_BASS24_ChannelPause(handle);
        else begin
            Result := t_BASS24_ChannelPause(handle);
        end;
    end;
end;

function BASS_ErrorGetCode: LongInt;
begin
    case BASS_Version of
        BASSVERSION23: Result := t_BASS23_ErrorGetCode;
        BASSVERSION24: Result := t_BASS24_ErrorGetCode;
        else begin
            Result := t_BASS24_ErrorGetCode;
        end;
    end;
end;

end.
